/* *****************************************************************************
 * decode/mcs/setup.c -- decode an MCS setup file.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_decode_mcs_setup:
 *	Decode settings.
 *
 *	@arg	handle		the handle to make.
 *	@arg	buffer		the buffer to read from.
 *	@arg	head		the pre-filled head to complete and use.
 *	@return				the error code (0 if ok).
 */

int g1m_decode_mcs_setup(g1m_mcsfile_t **handle, g1m_buffer_t *buffer,
	g1m_mcshead_t *head)
{
	/* make final head and file */
	int err = g1m_make_mcsfile(handle, head);
	if (err) return (err);
	unsigned char *content = (void*)(*handle)->content;
	READ(content, head->size)

	/* log angle setting */
	switch (content[0x13]) {
	case 0x00: log_info("[Angle] Degrees mode"); break;
	case 0x01: log_info("[Angle] Radians mode"); break;
	case 0x02: log_info("[Angle] Gradians mode"); break;
	}

	/* log number mode */
	switch (content[0x17]) {
	case 0x00: log_info("[Number] Comp"); break;
	case 0x01: log_info("[Number] Binary"); break;
	case 0x07: log_info("[Number] Octal"); break;
	case 0x09: log_info("[Number] Decimal"); break;
	case 0x0F: log_info("[Number] Hexadecimal"); break;
	}

	/* axes setting */
	log_info("[Axes] %s", content[0x1D] ? "on" : "off");

	/* list file */
	log_info("[ListFile] #%hhu", content[0x2E]);

	/* no error! */
	return (0);
}
