/* *****************************************************************************
 * encode/mcs/program.c -- encode an MCS program.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_announce_mcs_program:
 *	Announce a program's size.
 *
 *	@arg	handle		the handle to make the size out of.
 *	@arg	sz			the size to contribute to.
 *	@return				the error code (0 if ok).
 */

int g1m_announce_mcs_program(const g1m_mcsfile_t *handle, size_t *sz)
{
	*sz += sizeof(struct mcs_programheader) + handle->head.size;
	return (0);
}

/**
 *	g1m_encode_mcs_program:
 *	Encode a program.
 *
 *	@arg	handle		the handle to encode.
 *	@arg	buffer		the buffer to write to.
 *	@return				the error code (0 if ok).
 */

int g1m_encode_mcs_program(const g1m_mcsfile_t *handle, g1m_buffer_t *buffer)
{
	/* write the header */
	struct mcs_programheader hd = {};
	memcpy(hd.password, handle->head.password, strlen(handle->head.password));
	DWRITE(hd)

	/* write the program content */
	WRITE(handle->content, handle->head.size)

	/* no error! */
	return (0);
}
